#Region "License"
' SNES.Cart - SNES Cart handler (WIP)
' Copyright (C) 2006  TFG (tfg@panicus.org)

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#End Region

#Region "Comments/Credits"
' (1) All SNES cart information & data formats taken from
'     SNES Kart 1.6 Copyright (c) 1995-1996 DiskDude
'     http://www.classicgaming.com/EPR/super/sneskart.html
#End Region

Imports System.IO

Public Class Cart

#Region "SNES.Cart PRIVATE"
    ' Hardcoded offsets
    Private Const SZ_HEADER = &H200
    Private Const OFS_CHECKSUM_LOROM = &H7FDC
    Private Const OFS_CHECKSUM_HIROM = &HFFDC

    ' Properties' variables and such
    Dim fData As FileStream
    Dim p_hasHeader As Boolean
    Dim p_loRom As Boolean
    Dim p_hiRom As Boolean

    Private Sub checkLoHiRom()
        ' It's highly improbable that the rom's checksum
        ' could be 0xFFFF. That said, ignore 0xFFFFs

        Dim a As Byte, b As Byte, c As Byte, d As Byte, e As Integer
        If p_hasHeader Then e = SZ_HEADER Else c = 0

        ' LoROM
        fData.Seek(OFS_CHECKSUM_LOROM + e, SeekOrigin.Begin)
        a = fData.ReadByte() : b = fData.ReadByte()
        c = fData.ReadByte() : d = fData.ReadByte()
        If Not ((a = &HFF) And (b = &HFF)) Then
            p_loRom = (((a Or c) = &HFF) And ((b Or d) = &HFF))
        Else
            p_loRom = False
        End If

        ' HiROM
        fData.Seek(OFS_CHECKSUM_HIROM + e, SeekOrigin.Begin)
        a = fData.ReadByte() : b = fData.ReadByte()
        c = fData.ReadByte() : d = fData.ReadByte()
        If Not ((a = &HFF) And (b = &HFF)) Then
            p_hiRom = (((a Or c) = &HFF) And ((b Or d) = &HFF))
        Else
            p_hiRom = False
        End If
    End Sub

    Private Sub readProperties()
        ' LoROM/HiROM check
        checkLoHiRom()
        If ((Not p_loRom) And (Not p_hiRom)) Then
            p_hasHeader = True
            checkLoHiRom()
        End If
    End Sub

#End Region

#Region "SNES.Cart PUBLIC"
    ReadOnly Property hasHeader()
        Get
            Return p_hasHeader
        End Get
    End Property

    ReadOnly Property isLoROM()
        Get
            Return p_loRom
        End Get
    End Property

    ReadOnly Property isHiROM()
        Get
            Return p_hiRom
        End Get
    End Property

    Public Sub New(ByVal fileName As String)
        Try
            fData = New FileStream(fileName, FileMode.Open, FileAccess.Read)
        Catch ex As Exception
            Throw New Exception("An error occurred while trying to load the file.", ex)
        End Try

        ' Filesize check
        If (fData.Length < &HFFFF) Then Throw New Exception("Invalid file size.")

        readProperties()
        fData.Close()
    End Sub

    Public Sub New(ByRef Data As FileStream)
        fData = Data

        ' Filesize check
        If (fData.Length < &HFFFF) Then Throw New Exception("Invalid file size.")

        readProperties()
    End Sub
#End Region

End Class
